<?php
namespace lib\db;

use core;
use core\MYSQL;

class Auth
{
	public function __construct() 
    {
        $this->table = "auth";
		$settings = core\getSettings("../lib"); 
        $db=core\getVal($settings,"db");
        $this->sql = new MYSQL($db);
        $this->build();
    }

    function  build()
	{
		$fields=[];
		array_push($fields,"user_id INT(64) UNSIGNED  DEFAULT NULL");
		array_push($fields,"hash VARCHAR(256) NOT NULL");
		array_push($fields,"start TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP");
		array_push($fields,"end TIMESTAMP  DEFAULT NULL");
		$res = $this->sql-> buildTable($this->table,$fields);
        $res = json_encode($res);

        // print_r("//".$res.PHP_EOL);
    }

	function prepareData($input=[])
    {
        if(!$input) return $input;
        
        $data = core\getVal($input,"data",[]);
		
        $cols =  core\getVal($input,"cols", ["id","user_id","hash","end"]);
		// print_r($cols);

		$rowData = array_filter($data,	function ($key) use ($cols) {return in_array($key, $cols);},	ARRAY_FILTER_USE_KEY);

		foreach($cols as $key)
		{
			unset($data[$key]);
		}
        return $rowData;
    }

	function get($input=[])
    {
        $res = [];

		$auth = $this->getValidAuth($input);


		$hash = core\getVal($auth,'hash',false);
	
		if(!$hash)
		{
			$new_auth = $this->save($input);

			// print_r($new_auth);
			$data = core\getVal($new_auth,'data');
			$hash = core\getVal($data,'hash');
		}
		$res["data"]=["hash"=>$hash];
		return $res;


    }

	function save ($input = [])
	{
		$data["user_id"] = core\getVal($input,'user_id',0);
		$data["hash"] = core\gen_uuid();
		$input["data"] = $data;
		

        $input = $this->prepareData($input);

        $req = 
        [
            "table" => $this->table,
            "data" => $input,
        ];

		return $this->sql->save($req);
	}

	function getValidAuth($input=[])
	{
		$id =  core\getVal($input,'user_id',-1);
		$hash =  core\getVal($input,'hash');

		$req["where"] = 
		"
			WHERE
			(
				user_id = $id 
				OR 
				hash = '$hash' 
			)
			AND end is NULL
			AND TIMESTAMPDIFF(MINUTE,start,CURRENT_TIMESTAMP())<24
		";

        $req["table"] =$this->table;
        $req["limit"] = 1;
        $rows = $this->sql->getRows($req);
		
        $data = core\getVal($rows,'data',[]);
		
        $item = core\getVal($data,array_key_first($data));
		
		return $item;
	}

	function endAuth($input=[])
	{
		$data["end"] = date("Y-m-d H:i:s");
		$hash = core\getVal($input,'hash');

        $req = 
        [
            "data" => $data,
			"where" => "WHERE hash = '$hash'"
        ];

		$res = $this->edit($req);
		
		// print_r($res);
		return $res;
	}

	function edit($req=[])
	{
		if(!$req) $req = [];

		$res = [];
		$req["table"] = $this->table; 
		$res = $this->sql->edit($req);
		
		return $res;
	}
    
}