<?php
namespace lib\db;

include_once("../lib/f_proxy.php");
include("./link.php");

p_include("core/php/mysql2.php","");
p_include("core/php/api.php","");
p_include("core/php/util.php","");

use core;
use core\API;
use core\MYSQL;

abstract class Base extends API
{
    public $sql;
    public $table;
	public $link;

    public function __construct($table,$types=[]) 
    {
        $this->table = $table;
        $settings = core\getSettings("../lib"); 
        $db=core\getVal($settings,"db");
        $this->sql = new MYSQL($db);
		$this->link = new Link($table,$types);

        $this->build();
        $this->_run();
    }

    function  build()
	{
		$fields=[];
		array_push($fields,"name VARCHAR(256) NOT NULL");
		array_push($fields,"data JSON");
		$res = $this->sql-> buildTable($this->table,$fields);
        $res = json_encode($res);

        // print_r("//".$res.PHP_EOL);
    }

    function prepareData($input)
    {
        if(!$input) return $input;
        
        $data = core\getVal($input,"data");

        unset($data["type"]);

        $cols =  core\getVal($input,"cols", ["id","name"]);

		$rowData = array_filter($data,	function ($key) use ($cols) {return in_array($key, $cols);},	ARRAY_FILTER_USE_KEY);
		foreach($cols as $key)
		{
			unset($data[$key]);
		}
		$rowData["data"] = json_encode($data);
        return $rowData;
    }

    function save($input=[])
    {
        $res = [];

        
        $input = $this->prepareData($input);

        $req = 
        [
            "table" => $this->table,
            "data" => $input,
        ];

        $res = $this->sql->save($req);

        $data = core\getVal($res,'data',[]);
        $json = core\getVal($data,'data',"");
        $res["data"]["data"] = json_decode($json);
        return core\getVal($res,'data',[]);
    }

    function link_add($input = [])
	{
		return $this->link->save($input);
	}

	function link_remove($input = [])
	{
		return $this->link->remove($input);
	}

    function list($input=[])
    {
        $id = core\getVal($input,"entity_id",0);
		$type = core\getVal($input,"entity_type","");
		$quote_id = core\getVal($input,"quote_id",0);
		
        $where = "";
		if($id && $type && $type==$this->table)
		{
			$req = [];
			$req["child_type"] = $type;
			$req["child_id"] = $id;
			$res_ = $this->link->getParents($req);
	
			$ids = [];
			$ids[]= $id;
			if($res_)
			{
				$found = core\getVal($res_,"found",0);
				foreach($found as $parent)
				{
					$ids[] = core\getVal($parent,"id",0);
				}
			}

			$ids = implode(",",$ids);

            $where= "WHERE id NOT IN ($ids)";
            
		} 

        if($quote_id) $where .= " AND quote_id = $quote_id";

        $input["where"] = $where;
        $table = $this->table;
        $input["table"] =$table;
        $input["fields"] =["*","'$table' as type"];
        
        return $this->sql->getRows($input);
    }
    function view($input=[])
    {
        $id = core\getVal($input,"id");
        $where = core\getVal($input,"where");
        if(!$where && $id)
        {
            $req["where"] = "WHERE id = $id";
        } 
        else $req["where"] = $where;


        $req["table"] =$this->table;
        $req["limit"] = 1;
        $res = $this->sql->getRows($req);

        return core\getVal($res,'data',[]);

    }
    function remove($input=[])
    {
        $req = $input;
        $req["table"] = $this->table;
        $res = $this->sql->delete($req);
        return core\getVal($res,'data',[]);

    }
   
}
